@@ -26,7 +26,9 @@
 #                   Library imports                   #
 #-----------------------------------------------------#
 # External libraries
-from tensorflow.keras.layers import Activation, BatchNormalization, Dropout, LeakyReLU
+import tensorflow_addons as tfa
+
+from tensorflow.keras.layers import Activation, Dropout, LeakyReLU
 from tensorflow.keras.layers import Conv2D, MaxPooling2D, Conv2DTranspose
 from tensorflow.keras.layers import Conv3D, MaxPooling3D, Conv3DTranspose
 from tensorflow.keras.layers import Input, concatenate
@@ -50,18 +52,18 @@ class Architecture(Abstract_Architecture):
     #                Initialization               #
     #---------------------------------------------#
     def __init__(self, activation='softmax', conv_layer_activation='lrelu',
-                 batch_normalization=True, batch_normalization_params=None,
+                 instance_normalization=True, instance_normalization_params=None,
                  dropout=0, pooling=(1, 2, 2)):
         # Parse parameter
-        if batch_normalization_params is None:
-            batch_normalization_params = {'momentum': 0.1, 'epsilon': 1e-5}
+        if instance_normalization_params is None:
+            instance_normalization_params = {'epsilon': 1e-5}
         self.activation = activation
         # Parse activation layer
         if conv_layer_activation == "lrelu":
             self.conv_layer_activation = LeakyReLU(alpha=0.1)
         # Batch normalization settings
-        self.ba_norm = batch_normalization
-        self.ba_norm_params = batch_normalization_params
+        self.inst_norm = instance_normalization
+        self.inst_norm_params = instance_normalization_params
         # Dropout params
         self.dropout = dropout
         # Adjust pooling step
@@ -85,18 +87,18 @@ class Architecture(Abstract_Architecture):
         for i in range(0, len(self.feature_map)):
             neurons = self.feature_map[i]
             cnn_chain = conv_layer_2D(cnn_chain, neurons, self.conv_layer_activation,
-                                      self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                      self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
             cnn_chain = conv_layer_2D(cnn_chain, neurons, self.conv_layer_activation,
-                                      self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                      self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
             contracting_convs.append(cnn_chain)
             cnn_chain = MaxPooling2D(pool_size=(2, 2))(cnn_chain)
 
         # Middle Layer
         neurons = self.feature_map[-1]
         cnn_chain = conv_layer_2D(cnn_chain, neurons, self.conv_layer_activation,
-                                  self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                  self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
         cnn_chain = conv_layer_2D(cnn_chain, neurons, self.conv_layer_activation,
-                                  self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                  self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
 
         # Expanding Layers
         for i in reversed(range(0, len(self.feature_map))):
@@ -105,9 +107,9 @@ class Architecture(Abstract_Architecture):
                                         padding='same')(cnn_chain)
             cnn_chain = concatenate([cnn_chain, contracting_convs[i]], axis=-1)
             cnn_chain = conv_layer_2D(cnn_chain, neurons, self.conv_layer_activation,
-                                      self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                      self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
             cnn_chain = conv_layer_2D(cnn_chain, neurons, self.conv_layer_activation,
-                                      self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                      self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
 
         # Output Layer
         conv_out = Conv2D(n_labels, (1, 1), activation=self.activation)(cnn_chain)
@@ -131,9 +133,9 @@ class Architecture(Abstract_Architecture):
         # First contracting layer
         neurons = self.feature_map[0]
         cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                  self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                  self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
         cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                  self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                  self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
         contracting_convs.append(cnn_chain)
         cnn_chain = MaxPooling3D(pool_size=self.pooling)(cnn_chain)
 
@@ -141,18 +143,18 @@ class Architecture(Abstract_Architecture):
         for i in range(1, len(self.feature_map)):
             neurons = self.feature_map[i]
             cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                      self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                      self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
             cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                      self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                      self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
             contracting_convs.append(cnn_chain)
             cnn_chain = MaxPooling3D(pool_size=(2, 2, 2))(cnn_chain)
 
         # Middle Layer
         neurons = self.feature_map[-1]
         cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                  self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                  self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
         cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                  self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                  self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
 
         # Expanding Layers except last layer
         for i in reversed(range(1, len(self.feature_map))):
@@ -161,9 +163,9 @@ class Architecture(Abstract_Architecture):
                                         padding='same')(cnn_chain)
             cnn_chain = concatenate([cnn_chain, contracting_convs[i]], axis=-1)
             cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                      self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                      self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
             cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                      self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                      self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
 
         # Last expanding layer
         neurons = self.feature_map[0]
@@ -171,9 +173,9 @@ class Architecture(Abstract_Architecture):
                                     padding='same')(cnn_chain)
         cnn_chain = concatenate([cnn_chain, contracting_convs[0]], axis=-1)
         cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                  self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                  self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
         cnn_chain = conv_layer_3D(cnn_chain, neurons, self.conv_layer_activation,
-                                  self.ba_norm, self.ba_norm_params, self.dropout, strides=1)
+                                  self.inst_norm, self.inst_norm_params, self.dropout, strides=1)
 
         # Output Layer
         conv_out = Conv3D(n_labels, (1, 1, 1), activation=self.activation)(cnn_chain)
@@ -186,13 +188,13 @@ class Architecture(Abstract_Architecture):
 #                   Subroutines 2D                    #
 #-----------------------------------------------------#
 # Convolution layer
-def conv_layer_2D(input, neurons, activation, ba_norm, ba_norm_params, dropout, strides=1):
+def conv_layer_2D(input, neurons, activation, inst_norm, inst_norm_params, dropout, strides=1):
     conv = Conv2D(neurons, (3, 3), padding='same', strides=strides)(input)
 
     if dropout:
         conv = Dropout(dropout)(conv)
-    if ba_norm:
-        conv = BatchNormalization(**ba_norm_params)(conv)
+    if inst_norm:
+        conv = tfa.layers.InstanceNormalization(**inst_norm_params)(conv)
 
     return Activation(activation)(conv)
 
@@ -200,12 +202,12 @@ def conv_layer_2D(input, neurons, activation, ba_norm, ba_norm_params, dropout,
 #                   Subroutines 3D                    #
 #-----------------------------------------------------#
 # Convolution layer
-def conv_layer_3D(input, neurons, activation, ba_norm, ba_norm_params, dropout, strides=1):
+def conv_layer_3D(input, neurons, activation, inst_norm, inst_norm_params, dropout, strides=1):
     conv = Conv3D(neurons, (3, 3, 3), padding='same', strides=strides)(input)
 
     if dropout:
         conv = Dropout(dropout)(conv)
-    if ba_norm:
-        conv = BatchNormalization(**ba_norm_params)(conv)
+    if inst_norm:
+        conv = tfa.layers.InstanceNormalization(**inst_norm_params)(conv)
 
     return Activation(activation)(conv)
